==> CheckListProce-1.2.0-py3-none-any.whl: CheckListProce/CRFChecklistFunc.py <==
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 15 14:07:59 2023

@author: QianYang
# """
import pandas as pd
import pdfplumber
import openpyxl.styles as styles
from openpyxl.utils.dataframe import dataframe_to_rows
from docx import Document  

def getFormName(page_text,lable_formName,lable_subj = None):
    lines = page_text.splitlines()
    if lable_subj:
        output = None
        trigger = 0
        for line in lines:
            if lable_subj in line:
                trigger = 1
                continue
            if trigger == 1:
                trigger = 2
                continue
            if trigger == 2:
                output = line.strip()
                break
        if output != None:
            if lable_formName in output:
                key, value = output.split(lable_formName)
                formName = key.strip()
            else:
                formName = output
            return formName
    else:
        for line in lines:  
            if lable_formName in line:  
                parts = line.split(lable_formName, 1) 
                if len(parts) > 1:   
                    formName = parts[1].strip()  
                    return formName    
    return None
    
# def GetFormName_Rave(page_text, label_formName):  
#     lines = page_text.splitlines()  
#     for line in lines:  
#         if label_formName in line:  
#             parts = line.split(label_formName, 1)  # 分割一次，保留分隔符之后的部分  
#             if len(parts) > 1:  
#                 # 移除可能的前导空格  
#                 formName = parts[1].strip()  
#                 return formName  
#     return None
    
def simplifyNumList(num_list):
    ranges = []
    range_out = []
    start = num_list[0]
    end = num_list[0]
    for num in num_list[1:]:
        if num == end + 1:
            end = num
        else:
            ranges.append((start, end))
            start = num
            end = num
    ranges.append((start, end))
    for start,end in ranges:
        if start != end:
            range_out.append(str(start) + "-" + str(end))
        else:
            range_out.append(str(start))
    return range_out

def getFormList(path,for_rave):
    crf = None
    Lable_begin = "Protocol #"
    lable_formName = "dataset ="
    lable_formName_Rave = "Form:"
    lable_subj = "Site ID"
    
    with pdfplumber.open(path) as crf:
        Form_Dic = {}
        #从PDF取page
        for page in crf.pages: 
            if not for_rave:
                if Lable_begin in page.extract_text():             
                    #满足条件就把PDF header 和 page 添加到字典
                    formName = getFormName(page.extract_text(), lable_formName,lable_subj)
                    Form_Dic.setdefault(formName, []).append(page.page_number)
            else:
                if page.page_number>1:
                    formName = getFormName(page.extract_text(), lable_formName_Rave)
                    Form_Dic.setdefault(formName, []).append(page.page_number)
                    # print("Page finish: " + str(page.page_number) + "  " + formName)
                
        if Form_Dic:
            #对字典遍历，将页码变成页码范围
            for key,value in Form_Dic.items():
                Form_Dic[key] =",".join((simplifyNumList(value)))
                
        # #将字典的header和page整合起来，变成一个列表
        # Form_List = [f"{key}\n{value}" for key, value in Form_Dic.items()]
        return Form_Dic
    
def OutFile( folder_path, *args):
    with pd.ExcelWriter(folder_path, engine='openpyxl') as writer:
        workbook = writer.book
        sheets = []
        font = styles.Font(name='Arial', size=11)
        for i in range(len(args)//2):
            input_Dic = args[2*i]
            sheetName = args[2*i+1]
            dict_list = [f"{key}\n{value}" for key, value in input_Dic.items()]
            df = pd.DataFrame(dict_list, columns=['CRF page*'])
            if i == 0:
                df['Dataset' + "\n" + '(Question, OID, Name, Datatype, Code list, SAS Field)'], df['Codelist' + "\n" 
                    + '(Datatype, Label, Code, Referenced Variable)'], df['Variable Name' + "\n" 
                    + '(Label, Datatype, Length, Format)'], df['Result'],df['Initial'] = None, None, None, None, None
            else:
                df['Dataset' + "\n" + '(Question, OID, Name, Datatype, Code list, SAS Field)'], df['Codelist' + "\n" 
                    + '(Datatype, Label, Code, Referenced Variable)'], df['Variable Name' + "\n" 
                    + '(Label, Datatype, Length, Format)'], df['Spelling check'], df['Result'],df['Initial'] = None, None, None, None, None, None
            sheet = workbook.create_sheet(title=sheetName)
            sheets.append(sheet)
            for row in dataframe_to_rows(df, index=False, header=True):
                sheet.append(row)        
            for col in sheet.columns:
                for cell in col:
                    cell.font = font
        for sheet in sheets:
            workbook.active = workbook.index(sheet)
        workbook.close()

def OutFile_Docx(*args):
    for i in range(len(args)//2):
        doc = Document(args[2*i+1])
        dic = args[2*i]
        if doc.tables:
            table = doc.tables[1]        
            # while len(table.rows) > len(dic)+1:
            #     table.remove_row(-1)
            for index,(key,value)in enumerate(dic.items()):
                if index+2 > len(table.rows):
                    row_cells = table.add_row().cells
                else:
                    row_cells = table.rows[index+1].cells 
                row_cells[0].text = key + "\n" + value            
        doc.save(args[2*i+1])

#以下代码本地测试时使用
import tkinter as tk
from tkinter import filedialog
def getFileLoc(*files):
    root = tk.Tk()
    root.attributes('-topmost', 1)
    root.withdraw()
    file_paths = []
    if isinstance(files[-1],bool):
        path = filedialog.askdirectory(title=files[0]) +'/CKL.xlsx'
        file_paths.append(path)
    else:
        for file in files:
            file_path = filedialog.askopenfilename(title=file)
            file_paths.append(file_path)
    return tuple(file_paths)

==> CheckListProce-1.5.0-py3-none-any.whl: CheckListProce/CRFChecklistFunc.py <==
# -*- coding: utf-8 -*-
"""
Created on Fri Dec 15 14:07:59 2023

@author: QianYang
# """
import pandas as pd
import pdfplumber
import openpyxl.styles as styles
from openpyxl.utils.dataframe import dataframe_to_rows
from docx import Document  
import os

def getFormName(page_text,lable_formName,lable_subj = None):
    lines = page_text.splitlines()
    if lable_subj:
        output = None
        trigger = 0
        for line in lines:
            if lable_subj in line:
                trigger = 1
                continue
            if trigger == 1:
                trigger = 2
                continue
            if trigger == 2:
                output = line.strip()
                break
        if output != None:
            if lable_formName in output:
                key, value = output.split(lable_formName)
                formName = key.strip()
            else:
                formName = output
            return formName
    else:
        for line in lines:  
            if lable_formName in line:  
                parts = line.split(lable_formName, 1) 
                if len(parts) > 1:   
                    formName = parts[1].strip()  
                    return formName    
    return None
    
# def GetFormName_Rave(page_text, label_formName):  
#     lines = page_text.splitlines()  
#     for line in lines:  
#         if label_formName in line:  
#             parts = line.split(label_formName, 1)  # 分割一次，保留分隔符之后的部分  
#             if len(parts) > 1:  
#                 # 移除可能的前导空格  
#                 formName = parts[1].strip()  
#                 return formName  
#     return None
    
def simplifyNumList(num_list):
    ranges = []
    range_out = []
    start = num_list[0]
    end = num_list[0]
    for num in num_list[1:]:
        if num == end + 1:
            end = num
        else:
            ranges.append((start, end))
            start = num
            end = num
    ranges.append((start, end))
    for start,end in ranges:
        if start != end:
            range_out.append(str(start) + "-" + str(end))
        else:
            range_out.append(str(start))
    return range_out

def getFormList(path,for_rave):
    crf = None
    Lable_begin = "Protocol #"
    lable_formName = "dataset ="
    lable_formName_Rave = "Form:"
    lable_subj = "Site ID"
    
    with pdfplumber.open(path) as crf:
        Form_Dic = {}
        #从PDF取page
        for page in crf.pages: 
            if not for_rave:
                if Lable_begin in page.extract_text():             
                    #满足条件就把PDF header 和 page 添加到字典
                    formName = getFormName(page.extract_text(), lable_formName,lable_subj)
                    Form_Dic.setdefault(formName, []).append(page.page_number)
            else:
                if page.page_number>1:
                    formName = getFormName(page.extract_text(), lable_formName_Rave)
                    Form_Dic.setdefault(formName, []).append(page.page_number)
                    # print("Page finish: " + str(page.page_number) + "  " + formName)
                
        if Form_Dic:
            #对字典遍历，将页码变成页码范围
            for key,value in Form_Dic.items():
                Form_Dic[key] =",".join((simplifyNumList(value)))
                
        # #将字典的header和page整合起来，变成一个列表
        # Form_List = [f"{key}\n{value}" for key, value in Form_Dic.items()]
        return Form_Dic
    
def OutFile( folder_path, *args):
    with pd.ExcelWriter(folder_path, engine='openpyxl') as writer:
        workbook = writer.book
        sheets = []
        font = styles.Font(name='Arial', size=11)
        for i in range(len(args)//2):
            input_Dic = args[2*i]
            sheetName = args[2*i+1]
            dict_list = [f"{key}\n{value}" for key, value in input_Dic.items()]
            df = pd.DataFrame(dict_list, columns=['CRF page*'])
            if i == 0:
                df['Dataset' + "\n" + '(Question, OID, Name, Datatype, Code list, SAS Field)'], df['Codelist' + "\n" 
                    + '(Datatype, Label, Code, Referenced Variable)'], df['Variable Name' + "\n" 
                    + '(Label, Datatype, Length, Format)'], df['Result'],df['Initial'] = None, None, None, None, None
            else:
                df['Dataset' + "\n" + '(Question, OID, Name, Datatype, Code list, SAS Field)'], df['Codelist' + "\n" 
                    + '(Datatype, Label, Code, Referenced Variable)'], df['Variable Name' + "\n" 
                    + '(Label, Datatype, Length, Format)'], df['Spelling check'], df['Result'],df['Initial'] = None, None, None, None, None, None
            sheet = workbook.create_sheet(title=sheetName)
            sheets.append(sheet)
            for row in dataframe_to_rows(df, index=False, header=True):
                sheet.append(row)        
            for col in sheet.columns:
                for cell in col:
                    cell.font = font
        for sheet in sheets:
            workbook.active = workbook.index(sheet)
        workbook.close()

def OutFile_Docx(folder_path,*args):
    for i in range(len(args)//2):
        dic = args[2*i]
        file_path = args[2*i+1]
        doc = Document(file_path)
        if doc.tables:
            table = doc.tables[1]        
            # while len(table.rows) > len(dic)+1:
            #     table.remove_row(-1)
            for index,(key,value)in enumerate(dic.items()):
                if index+2 > len(table.rows):
                    row_cells = table.add_row().cells
                else:
                    row_cells = table.rows[index+1].cells 
                row_cells[0].text = key + "\n" + value
        save_path = os.path.join(folder_path, os.path.basename(file_path)) 
        doc.save(save_path)

#以下代码本地测试时使用
import tkinter as tk
from tkinter import filedialog
def getFileLoc(*files):
    root = tk.Tk()
    root.attributes('-topmost', 1)
    root.withdraw()
    file_paths = []
    if isinstance(files[-1],bool):
        path = filedialog.askdirectory(title=files[0])
        file_paths.append(path)
    else:
        for file in files:
            file_path = filedialog.askopenfilename(title=file)
            file_paths.append(file_path)
    return tuple(file_paths)
